use std::collections::HashMap;
use reqwest::header::HeaderMap;
use crate::models::{Submission, SubmissionRequest};
use anyhow::bail;

const FA_API_BASE_ORIGIN: &str = "https://furaffinity-api.herokuapp.com";

pub struct Client {
    request_client: reqwest::Client,
    cookies: HashMap<String, String>,
    bbcode: bool
}

impl Client {
    pub fn new(cookies: Option<HashMap<String, String>>, bbcode: Option<bool>) -> anyhow::Result<Self> {
        let headers = HeaderMap::from_iter(vec![
            (reqwest::header::ACCEPT, "application/json".try_into()?),
            (reqwest::header::CONTENT_TYPE, "application/json".try_into()?)
        ].into_iter());
        let request_client = reqwest::ClientBuilder::new()
            .default_headers(headers)
            .build()?;

        Ok(Self {
            request_client,
            cookies: cookies.unwrap_or_default(),
            bbcode: bbcode.unwrap_or_default()
        })
    }

    pub async fn get_submission<S>(&mut self, submission_id: S) -> anyhow::Result<Submission> where S: Into<String> {
        let url = format!("{FA_API_BASE_ORIGIN}/submission/{}/", submission_id.into());
        let cookies = self.cookies.clone()
            .into_iter()
            .map(|cookie| cookie.into())
            .collect();

        let res = self.request_client
            .post(url)
            .json(&SubmissionRequest {
                cookies,
                bbcode: self.bbcode.clone()
            })
            .send().await?;

        if res.status() != reqwest::StatusCode::OK {
            bail!(res.status());
        };
        Ok(res.json().await?)
    }
}
